#include "v3.h"
#include "m3x3.h"
#include <math.h>
#include <float.h>

V3::V3(float x, float y, float z) {

  xyz[0] = x;
  xyz[1] = y;
  xyz[2] = z;

}

V3 V3::operator+(V3 op1) {

  V3 ret;
  ret.xyz[0] = xyz[0] + op1.xyz[0];
  ret.xyz[1] = xyz[1] + op1.xyz[1];
  ret.xyz[2] = xyz[2] + op1.xyz[2];

  return ret;

}

V3 V3::operator-(V3 op1) {

  V3 ret;
  ret.xyz[0] = xyz[0] - op1.xyz[0];
  ret.xyz[1] = xyz[1] - op1.xyz[1];
  ret.xyz[2] = xyz[2] - op1.xyz[2];

  return ret;

}

float V3::operator*(V3 op1) {

  float ret;
  ret = xyz[0]*op1.xyz[0]+
    xyz[1]*op1.xyz[1]+xyz[2]*op1.xyz[2];
  return ret;

}

V3 V3::operator^(V3 op1) {

  V3 ret;
  ret.xyz[0] = xyz[1]*op1.xyz[2]-xyz[2]*op1.xyz[1];
  ret.xyz[1] = xyz[2]*op1.xyz[0]-xyz[0]*op1.xyz[2];
  ret.xyz[2] = xyz[0]*op1.xyz[1]-xyz[1]*op1.xyz[0];

  return ret;

}

float& V3::operator[](int i) {

  return xyz[i];

}

ostream& operator<<(ostream &ostr, V3 v) {

  ostr << v[0] << " " << v[1] << " " << v[2] << " ";
  return ostr;

}

istream& operator>>(istream &istr, V3 &v) {

  istr >> v[0] >> v[1] >> v[2];
  return istr;

}

float V3::Length() {

  float l2 = (*this)*(*this);
  return sqrtf(l2);

}

V3 V3::operator*(float scf) {

  V3 ret = *this;
  ret.xyz[0] *= scf;
  ret.xyz[1] *= scf;
  ret.xyz[2] *= scf;
  return ret;
}

V3 V3::operator/(float scf) {

  V3 ret = *this;
  ret.xyz[0] /= scf;
  ret.xyz[1] /= scf;
  ret.xyz[2] /= scf;
  return ret;
}

void V3::SetFromColor(unsigned int color) {

  unsigned char *cpt = (unsigned char*) & color;
  xyz[0] = (float) (cpt[0]) / 255.0f;
  xyz[1] = (float) (cpt[1]) / 255.0f;
  xyz[2] = (float) (cpt[2]) / 255.0f;

}

unsigned int V3::GetColor() {

  unsigned char rgb[3];
  rgb[0] = (unsigned char)(xyz[0]*255.0f);
  rgb[1] = (unsigned char)(xyz[1]*255.0f);
  rgb[2] = (unsigned char)(xyz[2]*255.0f);
  unsigned int ret = 0xFF000000 + rgb[2]*256*256 +
    rgb[1]*256 + rgb[0];
  return ret;

}

AABB::AABB() {

  corners[0] = V3(FLT_MAX, FLT_MAX, FLT_MAX);
  corners[1] = V3(-FLT_MAX, -FLT_MAX, -FLT_MAX);

}

void AABB::AddPoint(V3 p) {

  for (int j = 0; j < 3; j++) {
    if (p[j] < corners[0][j])
      corners[0][j] = p[j];
    if (p[j] > corners[1][j])
      corners[1][j] = p[j];
  }

}

V3 V3::Normalized() {

  return (*this)/Length();

}
